const logger = require('./logger.js');
const cache = require('./cache');
const os = require('os');

function getHostname() {
    return os.hostname();
}

function getUptime() {
    return os.uptime();
}

function getLoadAverage(index) {
    var loadAverage = os.loadavg();
    if (index <= loadAverage.length) {
        return os[index];
    }
    return os.loadavg();
}

function getArchitecture() {
    return os.arch();
}

function getOperatingSystem() {
    return os.release();
}

function getType() {
    return os.type();
}

function getPlatform() {
    return os.platform();
}

function getKernelVersion() {
    return os.version();
}

function getCpuInfo() {
    return os.cpus();
}

function getMemoryInfo() {
    var free = os.freemem();
    var total = os.totalmem();
    return {
        total: total,
        free: free,
        used: total - free
    }
}

function getHostInfo() {
    var info = cache.getInfo();
    if (info != undefined) {
        return info;
    }
    info = {
        hostname: getHostname(),
        uptime: getUptime(),
        load: getLoadAverage(),
        arch: getArchitecture(),
        os: getOperatingSystem(),
        type: getType(),
        platform: getPlatform(),
        kernel: getKernelVersion(),
        cpu: getCpuInfo(),
        memory: getMemoryInfo()
    }
    cache.setInfo(info);
    return info;
}

module.exports = {
    getHostname,
    getUptime,
    getLoadAverage,
    getArchitecture,
    getOperatingSystem,
    getType,
    getPlatform,
    getKernelVersion,
    getCpuInfo,
    getMemoryInfo,
    getHostInfo
}